@@ -29,13 +29,13 @@ class UserAuthnMethod(object):
         self.srv = srv
 
     def __call__(self, *args, **kwargs):
-        raise NotImplemented
+        raise NotImplementedError
 
     def authenticated_as(self, **kwargs):
-        raise NotImplemented
+        raise NotImplementedError
 
     def verify(self, **kwargs):
-        raise NotImplemented
+        raise NotImplementedError
 
 
 def is_equal(a, b):
